/*****************************************************************************
*																			 *
* PROGRAM: CVME.CPP															 
*
* PURPOSE: Define a VME class that is used to talk to VME via bit3 interface.
*          Uses the Bit 3 Model 983 support software.
*
*
* Author:  B. Paul Padley, Rice University 1996
*
*
* Oct 28, 1998 - Major rewrite, B.Paul Padley:
*                     Revised to handle V2.2 of bit3 API
*
* July 3, 2000 - Add in the code by Alex Madorsky and Alex Atamanchuk
*                      that uses memory mapping.  Modify that code slightly
*					  to make more general purpose
****************************************************************************/
#ifndef _CONSOLE
//#include        "stdafx.h"
#endif
#include		<windows.h>
#define BT983
#include		<btapi.h>
#include		"cvme.h"

CVME::CVME(void){
}

void CVME::Initialize(bt_dev_t logicalDevice){
//
	//open the device
	Initialize(0,logicalDevice);
		


}
void CVME::Initialize(int unit, bt_dev_t logicalDevice){
//
	// Get the device name
	
	bt_dev_t ldevice = logicalDevice;

	//open the device
		
 	error_status = bt_open(&device,
		bt_gen_name(unit, ldevice, devName, BT_MAX_DEV_NAME),BT_RDWR);
	if(error_status == BT_SUCCESS){
			
	// clear any errors on the device

		bt_clrerr(device);
	}
//	}
	if(error_status != BT_SUCCESS){
#ifndef _CONSOLE      /*
		CString errormsg = "Could Not Initialize VME ";
		int response = AfxMessageBox(errormsg,MB_RETRYCANCEL,0);
		if(response == IDRETRY) Initialize(unit,ldevice); */
#endif
	}



}



void CVME::Clear(void){error_status = bt_clrerr(device);}

void CVME::Reset(void){error_status = bt_clrerr(device);}

bt_error_t CVME::GetStatus(void){ return error_status;}

CVME::~CVME(void){

	if(isMapped)bt_unmmap(device, lpMemMapBase, imageMapSize); 

	error_status = bt_close(device);

}

bt_error_t CVME::SetMemoryMap(bt_devaddr_t address, DWORD imageSize){

	if(isMapped)bt_unmmap(device, lpMemMapBase, imageMapSize); 

	imageMapSize = imageSize;

	void * memMapBase;
	
	error_status = bt_mmap( device, &memMapBase, address, imageSize, 
							BT_RDWR, BT_SWAP_NONE );
	isMapped = TRUE;
	lpMemMapBase = (BYTE *)memMapBase;

	return error_status;
}


void CVME::WriteMemMap(DWORD offset, BYTE* data, DWORD numOfBytes)
{
	if(dataSize != BT_WIDTH_D8){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D8);
		dataSize = BT_WIDTH_D8;
	}
	DWORD i;
	if(isMapped)for (i = 0; i <numOfBytes; i++) *((BYTE*)(lpMemMapBase + offset)) = data[i];
}

void CVME::WriteMemMap(DWORD offset, WORD data)  
{
	if(dataSize != BT_WIDTH_D16){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D16);
		dataSize = BT_WIDTH_D16;
	}

	if(isMapped) *((WORD*)(lpMemMapBase + offset)) = data;    
}

void CVME::WriteMemMap(DWORD offset, WORD* data, DWORD numOfWords)
{
	DWORD i;
	if(dataSize != BT_WIDTH_D16){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D16);
		dataSize = BT_WIDTH_D16;
	}
	if(isMapped)for (i = 0; i <numOfWords; i++) *((WORD*)(lpMemMapBase + offset)) = data[i];
}

void CVME::WriteMemMap(DWORD offset, DWORD* data, DWORD numOfDoubleWords)
{
	if(dataSize != BT_WIDTH_D32){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D32);
		dataSize = BT_WIDTH_D32;
	}
	DWORD i;
	if(isMapped)for (i = 0; i <numOfDoubleWords; i++) *((DWORD*)(lpMemMapBase + offset)) = data[i];
}

BYTE CVME::ReadMemMap(DWORD offset)
{
	if(dataSize != BT_WIDTH_D8){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D8);
		dataSize = BT_WIDTH_D8;
	}
	if(isMapped){
		return (*((BYTE*)(lpMemMapBase + offset)));
	}else{
		return 0;
	}
}	

void CVME::ReadMemMap(DWORD offset, WORD* data, DWORD numOfWords)
{
	DWORD i;
	if(dataSize != BT_WIDTH_D16){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D16);
		dataSize = BT_WIDTH_D16;
	}
	if(isMapped){
		for (i = 0; i <numOfWords; i++) data[i] = *((WORD*)(lpMemMapBase + offset));

	}else{
		data = NULL;
	}
}



void CVME::ReadMemMap(DWORD offset, BYTE* data, DWORD numOfBytes)
{
	DWORD i;
	if(dataSize != BT_WIDTH_D8){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D8);
		dataSize = BT_WIDTH_D8;
	}
	if(isMapped){
		for (i = 0; i <numOfBytes; i++)data[i] = (*((BYTE*)(lpMemMapBase + offset)));
	}else{
		data = NULL;
	}
}

void CVME::ReadMemMap(DWORD offset, DWORD* data, DWORD numOfDoubleWords)
{
	DWORD i;
	if(dataSize != BT_WIDTH_D32){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D32);
		dataSize = BT_WIDTH_D32;
	}
	if(isMapped){
		for (i = 0; i <numOfDoubleWords; i++)data[i] = (*((DWORD*)(lpMemMapBase + offset)));
	}else{
		data = NULL;
	}
}



bt_error_t CVME::Write(bt_devaddr_t address, DWORD * Buffer, DWORD numOfBytes){
	size_t numOfBytesWritten;
	if(dataSize != BT_WIDTH_D32){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D32);
		dataSize = BT_WIDTH_D32;
	}
	error_status = bt_write(device, Buffer, address, numOfBytes, &numOfBytesWritten);
	
	return error_status;
}

bt_error_t CVME::Write(bt_devaddr_t address, DWORD * lpBuffer, DWORD numOfBytes, DWORD blockSizeInBytes){
	size_t numOfBytesWritten;
	DWORD i;
	if(dataSize != BT_WIDTH_D32){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D32);
		dataSize = BT_WIDTH_D32;
	}


	for (i = numOfBytes; i > blockSizeInBytes; i -= blockSizeInBytes, lpBuffer += blockSizeInBytes)
	
		error_status = bt_write(device, lpBuffer, address, blockSizeInBytes, &numOfBytesWritten);

	if (i > 0) 	error_status = bt_write(device, lpBuffer, address, i, &numOfBytesWritten);

	return error_status;
}

bt_error_t CVME::Write(bt_devaddr_t address, WORD * lpBuffer, DWORD numOfBytes, DWORD blockSizeInBytes){
	size_t numOfBytesWritten;
	DWORD i;
	
	if(dataSize != BT_WIDTH_D16){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D16);
		dataSize = BT_WIDTH_D16;
	}


	for (i = numOfBytes; i > blockSizeInBytes; i -= blockSizeInBytes, lpBuffer += blockSizeInBytes)
	
		error_status = bt_write(device, lpBuffer, address, blockSizeInBytes, &numOfBytesWritten);

	if (i > 0) 	error_status = bt_write(device, lpBuffer, address, i, &numOfBytesWritten);

	return error_status;
}

bt_error_t CVME::Write(bt_devaddr_t address, BYTE * lpBuffer, DWORD numOfBytes, DWORD blockSizeInBytes){
	size_t numOfBytesWritten;
	DWORD i;
	
	if(dataSize != BT_WIDTH_D8){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D8);
		dataSize = BT_WIDTH_D8;
	}

	for (i = numOfBytes; i > blockSizeInBytes; i -= blockSizeInBytes, lpBuffer += blockSizeInBytes)
	
		error_status = bt_write(device, lpBuffer, address, blockSizeInBytes, &numOfBytesWritten);

	if (i > 0) 	error_status = bt_write(device, lpBuffer, address, i, &numOfBytesWritten);

	return error_status;
}



bt_error_t CVME::Write(bt_devaddr_t address, WORD * lpBuffer, DWORD numOfBytes){
	size_t numOfBytesWritten;
	if(dataSize != BT_WIDTH_D16){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D16);
		dataSize = BT_WIDTH_D16;
	}
	error_status = bt_write(device, lpBuffer, address, numOfBytes, &numOfBytesWritten);
	
	return error_status;
}
bt_error_t CVME::Write(bt_devaddr_t address, BYTE * lpBuffer, DWORD numOfBytes){
	size_t numOfBytesWritten;
	if(dataSize != BT_WIDTH_D8){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D8);
		dataSize = BT_WIDTH_D8;
	}
	error_status = bt_write(device, lpBuffer, address, numOfBytes, &numOfBytesWritten);
	
	return error_status;
}
bt_error_t CVME::Write(bt_devaddr_t address, DWORD Buffer){
	size_t numOfBytesWritten;
	if(dataSize != BT_WIDTH_D32){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D32);
		dataSize = BT_WIDTH_D32;
	}
	error_status = bt_write(device, &Buffer, address, 4, &numOfBytesWritten);
	
	return error_status;
}
bt_error_t CVME::Write(bt_devaddr_t address, WORD Buffer){
	size_t numOfBytesWritten;
	if(dataSize != BT_WIDTH_D16){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D16);
		dataSize = BT_WIDTH_D16;
	}
	error_status = bt_write(device, &Buffer, address, 2, &numOfBytesWritten);
	
	return error_status;
}
bt_error_t CVME::Write(bt_devaddr_t address, BYTE Buffer){
	size_t numOfBytesWritten;
	if(dataSize != BT_WIDTH_D8){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D8);
		dataSize = BT_WIDTH_D8;
	}
	error_status = bt_write(device, &Buffer, address, 1, &numOfBytesWritten);

	
	return error_status;
}

bt_error_t CVME::Read(bt_devaddr_t address, DWORD * lpBuffer, DWORD numOfBytes){
	size_t numOfBytesRead;
	if(dataSize != BT_WIDTH_D32){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D32);
		dataSize = BT_WIDTH_D32;
	}
	error_status = bt_read(device, lpBuffer, address, numOfBytes, &numOfBytesRead);

	return error_status;
}
bt_error_t CVME::Read(bt_devaddr_t address, WORD * lpBuffer, DWORD numOfBytes){
	size_t numOfBytesRead;
	if(dataSize != BT_WIDTH_D16){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D16);
		dataSize = BT_WIDTH_D16;
	}
	error_status = bt_read(device, lpBuffer, address, numOfBytes, &numOfBytesRead);

	return error_status;
}
bt_error_t CVME::Read(bt_devaddr_t address, BYTE * lpBuffer, DWORD numOfBytes){
	size_t numOfBytesRead;
	if(dataSize != BT_WIDTH_D8){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D8);
		dataSize = BT_WIDTH_D8;
	}
	error_status = bt_read(device, lpBuffer, address, numOfBytes, &numOfBytesRead);

	return error_status;
}
bt_error_t CVME::Read(bt_devaddr_t address, DWORD * lpBuffer){
	size_t numOfBytesRead;
	if(dataSize != BT_WIDTH_D32){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D32);
		dataSize = BT_WIDTH_D32;
	}
	error_status = bt_read(device, lpBuffer, address, 4, &numOfBytesRead);

	return error_status;
}
bt_error_t CVME::Read(bt_devaddr_t address, WORD * lpBuffer){
	size_t numOfBytesRead;
	if(dataSize != BT_WIDTH_D16){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D16);
		dataSize = BT_WIDTH_D16;
	}
	error_status = bt_read(device, lpBuffer, address, 2, &numOfBytesRead);

	return error_status;
}
bt_error_t CVME::Read(bt_devaddr_t address, BYTE * lpBuffer){
	size_t numOfBytesRead;
	if(dataSize != BT_WIDTH_D8){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D8);
		dataSize = BT_WIDTH_D8;
	}
	error_status = bt_read(device, lpBuffer, address, 1, &numOfBytesRead);
	return error_status;
}

bt_error_t CVME::Read(bt_devaddr_t address, BYTE * lpBuffer, DWORD numOfBytes, DWORD blockSizeInBytes){
	size_t numOfBytesRead;
	DWORD i;
	if(dataSize != BT_WIDTH_D8){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D8);
		dataSize = BT_WIDTH_D8;
	}

	for (i = numOfBytes; i > blockSizeInBytes; i -= blockSizeInBytes, lpBuffer += blockSizeInBytes)
	
		error_status = bt_read(device, lpBuffer, address, blockSizeInBytes, &numOfBytesRead);

	if (i > 0) 	error_status = bt_read(device, lpBuffer, address, i, &numOfBytesRead);
	return error_status;
}

bt_error_t CVME::Read(bt_devaddr_t address, WORD * lpBuffer, DWORD numOfBytes, DWORD blockSizeInBytes){
	size_t numOfBytesRead;
	DWORD i;
	if(dataSize != BT_WIDTH_D16){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D16);
		dataSize = BT_WIDTH_D16;
	}

	for (i = numOfBytes; i > blockSizeInBytes; i -= blockSizeInBytes, lpBuffer += blockSizeInBytes)
	
		error_status = bt_read(device, lpBuffer, address, blockSizeInBytes, &numOfBytesRead);

	if (i > 0) 	error_status = bt_read(device, lpBuffer, address, i, &numOfBytesRead);
	return error_status;
}

bt_error_t CVME::Read(bt_devaddr_t address, DWORD * lpBuffer, DWORD numOfBytes, DWORD blockSizeInBytes){
	size_t numOfBytesRead;
	DWORD i;
	if(dataSize != BT_WIDTH_D32){
		error_status = bt_set_info(device, BT_INFO_DATAWIDTH, BT_WIDTH_D32);
		dataSize = BT_WIDTH_D32;
	}

	for (i = numOfBytes; i > blockSizeInBytes; i -= blockSizeInBytes, lpBuffer += blockSizeInBytes)
	
		error_status = bt_read(device, lpBuffer, address, blockSizeInBytes, &numOfBytesRead);

	if (i > 0) 	error_status = bt_read(device, lpBuffer, address, i, &numOfBytesRead);
	return error_status;
}

void CVME::SetDMAThreshold(WORD dmaThreshold){
		bt_set_info(device, BT_INFO_DMA_THRESHOLD, dmaThreshold);
}


void CVME::SetAMOD(DWORD amod){
	error_status = bt_set_info(device, BT_INFO_PIO_AMOD, amod);
	error_status = bt_set_info(device, BT_INFO_DMA_AMOD, amod);
}
DWORD CVME::GetAMOD(void){
	DWORD amod;
	bt_error_t error_status = bt_get_info(device, BT_INFO_PIO_AMOD,&amod);
	return amod;
}

