/***********************************************************************
*  Karol Bunkowski 2002
*  kbunkow@fuw.edu.pl
*  Warsaw University
*	 
*  
*  
/***********************************************************************/

#include "stdafx.h"
#include "TTCviN.h"
#include "TVMEWinBit3.h"


int main(int argc, char* argv[])
{
	const bool internalOrbit = true;
//B-Go<0> settings
	const bool	enable0 = 1; // 1 = front pannel input disable
	const bool	synch0  = 0; // 0 = synchronous cycle
	const bool	single0 = 1; // 1 = repetitive mode
	const bool	fifo0   = 1; // 0 = don't look at fifo status	TTTCvi* TTC;
	
	const int inh0del = 0;
	const int inh0dur = 1;

	//B-Go<1> probably broken
//B-Go<2> settings
	const bool	enable2 = 1; // 1 = front pannel input disable
	const bool	synch2  = 0; // 0 = synchronous cycle
	const bool	single2 = 1; // 1 = repetitive mode
	const bool	fifo2   = 1; // 0 = don't look at fifo status	TTTCvi* TTC;

	const int inh2del = 50;
	const int inh2dur = 1;

			// CCB_cmd[5...0] is the TTC command[7....2] bits
	const int bx0 = 0x1 <<2 & 0xFC;        //0xFC = 11111100; 
	const int L1ARES = 0x3 <<2 & 0xFC; 
	const int DMB_HARD_RST = 0x12 <<2 & 0xFC;
	const int DMB_SOFT_RST = 0x1D <<2 & 0xFC; 
	const int CFEB_CAL0 = 0x14 <<2 & 0xFC;
	const int CFEB_CAL1 = 0x15 <<2 & 0xFC;
	const int CFEB_CAL2 = 0x16 <<2 & 0xFC;

	TVMEWinBit3* VME;
	TTTCvi* TTC;
	try {	
		VME = new TVMEWinBit3();
		TTC = new TTTCvi(VME,0xff1000); //0xff1000 - TTCvi board address
		
		TTC->Reset();
		cout<<"TTCvi reset\n";
		
		
		TTC->OrbitSignalSel(internalOrbit); 
		if(internalOrbit)
			cout<<"Orbit signal generated internally.\n\n";
		else
			cout<<"External orbit signal expected.\n\n";
		
		std::bitset<4> RetransmitFIFO;
			RetransmitFIFO.set(); //"1" = "not retrtansmit"
 		std::bitset<4> ResetFIFO;

//Bunch counter reset signal
		TTC->SetInhibit0(inh0del,inh0dur);
		cout<<"inhibit 0: delay= "<<inh0del<<",duration = "<<inh0dur<<endl;
		

		ResetFIFO[0]=true;
		TTC->ResetBGoFIFO(ResetFIFO);
		ResetFIFO[0]=false;
		cout<<"B-Go FIFO<0> reset\n"; 

		TTC->b_Go[0]->Settings(enable0, synch0, single0, fifo0,0);
		cout<<"setting B_Go<0> mode:\n";
		cout<<"enable = "<<enable0<<"\nsynch = "<<synch0<<"\nsingle = "<<single0<<"\nfifo = "<<fifo0<<"\n";

		cout<<"Checking B_Go<0> settings:\n";
		std::bitset<5> bg0Settings;
		bg0Settings = TTC->b_Go[0]->ReadSettings(); //Checking B_Go<0> settings
		
		if(bg0Settings[0] != enable0)
			cout<<"\n!!!!!!!!!!!!!!!! Bit 'enable' not set properly!!!!!!!!!!!!!\n";
		else
			cout<<"ok ";
		
		if(bg0Settings[1] != synch0)
			cout<<"\n!!!!!!!!!!!!!!!! Bit 'synch' not set properly!!!!!!!!!!!!!!\n";
		else
			cout<<"ok ";

		if(bg0Settings[2] != single0)
			cout<<"\n!!!!!!!!!!!!!!!! Bit 'single' not set properly!!!!!!!!!!!!!\n";
		else
			cout<<"ok ";

		if(bg0Settings[3] != fifo0)
			cout<<"\n!!!!!!!!!!!!!!!! Bit 'fifo' not set properly!!!!!!!!!!!!!!!\n";
		else
			cout<<"ok\n";
		
		RetransmitFIFO[0]=false;  
		TTC->RetransmiteBGoFIFO(RetransmitFIFO); //0 on CSR2<8> means retransmite B-Go FIFO<0> 

		TTC->b_Go[0]->WriteDataShort(1); // first bit of a broadcast command denotes BCR
		cout<<"B-Go FIFO<0> loaded with Bunch Counter Reset command"<<endl<<endl;
//End of Bunch counter reset signal	

//BX0 signal
		TTC->SetInhibit2(inh2del,inh2dur);
		cout<<"inhibit 2: delay= "<<inh2del<<",duration = "<<inh2dur<<endl;
		
		ResetFIFO[2]=true;
		TTC->ResetBGoFIFO(ResetFIFO);
		ResetFIFO[2]=false;
		cout<<"B-Go FIFO<2> reset\n"; 

		TTC->b_Go[2]->Settings(enable2, synch2, single2, fifo2,0);
		cout<<"setting B_Go<1> mode:\n";
		cout<<"enable = "<<enable2<<"\nsynch = "<<synch2<<"\nsingle = "<<single2<<"\nfifo = "<<fifo2<<"\n";

		cout<<"Checking B_Go<1> settings:\n";
		std::bitset<5> bg2Settings;
		bg2Settings = TTC->b_Go[2]->ReadSettings(); //Checking B_Go<1> settings
		
		if(bg2Settings[0] != enable2)
			cout<<"\n!!!!!!!!!!!!!!!! Bit 'enable' not set properly!!!!!!!!!!!!!\n";
		else
			cout<<"ok ";
		
		if(bg2Settings[1] != synch2)
			cout<<"\n!!!!!!!!!!!!!!!! Bit 'synch' not set properly!!!!!!!!!!!!!!\n";
		else
			cout<<"ok ";

		if(bg2Settings[2] != single2)
			cout<<"\n!!!!!!!!!!!!!!!! Bit 'single' not set properly!!!!!!!!!!!!!\n";
		else
			cout<<"ok ";

		if(bg2Settings[3] != fifo2)
			cout<<"\n!!!!!!!!!!!!!!!! Bit 'fifo' not set properly!!!!!!!!!!!!!!!\n";
		else
			cout<<"ok\n";
		
		RetransmitFIFO[2]=false;
		TTC->RetransmiteBGoFIFO(RetransmitFIFO); //0 on CSR2<8> means retransmite B-Go FIFO<0> 

		TTC->b_Go[2]->WriteDataShort(bx0);
		cout<<"B-Go FIFO<2> loaded with Bunch Crosing 0 command"<<endl<<endl;
//End of BX0 signal	
					
		cout<<"Press:"<<endl<<"l = L1A siggnal\n";
		cout<<"r = L1ARST signal"<<endl;
		cout<<"h = DMB_HARD_RST"<<endl;
		cout<<"s = DMB_SOFT_RST"<<endl;
		cout<<"0 = CFEB_CAL0"<<endl;
		cout<<"1 = CFEB_CAL1"<<endl;
		cout<<"2 = CFEB_CAL2"<<endl;
		cout<<"o = other"<<endl;
		cout<<"q = Quit"<<endl; 		
		char c='0';	

		while(c!='q')
		{
			c = _getch();  //if not in Visuall studio change on getch()
			
			switch(c)
			{
			 case 'l':
				{
					TTC->Trigger();
					cout<<"L1A sent"<<endl;
					break;
				}				
			 case 'r':
				{
					TTC->WriteShortAsynchronous(L1ARES);
					cout<<"L1ARES sent"<<endl;
					break;
				}
			 case 'h':
				{
					TTC->WriteShortAsynchronous(DMB_HARD_RST);
					cout<<"DMB_HARD_RST sent"<<endl;
					break;
				}
			 case 's':
				{
					TTC->WriteShortAsynchronous(DMB_SOFT_RST);
					cout<<"DMB_SOFT_RST sent"<<endl;
					break;
				}
			  case '0':
				{
					TTC->WriteShortAsynchronous(CFEB_CAL0);
					cout<<"CFEB_CAL0 sent"<<endl;
					break;
				}
			  case '1':
				{
					TTC->WriteShortAsynchronous(CFEB_CAL1);
					cout<<"CFEB_CAL1 sent"<<endl;
					break;
				}
			  case '2':
				{
					TTC->WriteShortAsynchronous(CFEB_CAL2);
					cout<<"CFEB_CAL2 sent"<<endl;
					break;
				}
		      case 'o':
				{
					cout<<"Insert code of command (in hexadecimal format) ";
					int command, command1;
					cin>>hex>>command;
					command1 = (command <<2 & 0xFC ); // CCB_cmd[5...0] is the TTC command[7....2] bits
					TTC->WriteShortAsynchronous(command1);
					cout<<"Command = "<<hex<<command<<" sent"<<endl;
					break;
				}

			}

		}

       }
       catch(EVME& err)
       {
        cout<<err.what()<<endl;
		cout<<"Press any key"<<endl;
		_getch();

       }
	return 0;
}

